// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com
// Apdo. 964-1250 Escaz, San Jos, Costa Rica 
// dedicado a Jess y Mara...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 15 febrero 2011
//                
// Mis dibujos estn en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY 
//
// "MesaTVC.inc" dibuja una mesa para terraza
// Sus dimensiones son ( sin deformaciones adicionales ):
// Alto                 = 535*L 
//             514+21*FactorGrueso si utiliza la macro
// Radio                = 325*L
//             RadioPrincipal si utiliza la macro         
// Su ubicacion primaria descansa sobre el plano XZ centrado en el origen.
//
// Para ponerlo en pantalla simplemente utilice la macro default
// #include "MesaTVC.inc"
// object { MTVC_MesaTerraza() }
// Si utiliza el FreeDOS, con la version 3.1 de PovRay
// #declare MTVC_POV3_1 = on;
// #include "MesaTVC.inc"
// object { MTVC_MesaTerraza() }
//
// Otra alternativa disponible es utilizar la macro:
// object { MTVC_MesaTerraza
//      ( RadioPrincipal, RadioMedio, FactorGrueso, 
//                TexturaMetal, MaterialVidrio ) } 
//      RadioPrincipal  : radio del vidrio
//      RadioMedio      : radio de la zona inferior
//      FactorGrueso    : 1.0 es default, amplifica o disminuye
//              el grueso de las barras
//      TexturaMetal    : Textura del objeto fundamental
//      MaterialVidrio  : tipo material para describir el vidrio
//
// Para accesar a las siguientes texturas basicas que
// no deberian modificarse, las variables son
// #declare MTVC_TexturaMetal
// #declare MTVC_MaterialVidrio
// Si usted desea modificar la textura default, utilice 
// las siguientes variables
// #declare MTVC_TexturaMetalDefault
// #declare MTVC_MaterialVidrioDefault
// Por ejemplo 
// #include "TrasteroSMB.inc"
// #declare CustomTextura               = texture       {
//   pigment            { Cyan }
// }
// #declare TSMB_TexturaMetalDefault    = texture       {
//   average
//   texture_map   {
//      [ 0.5 MTVC_TexturaMetal        ]
//      [ 0.5 CustomTextura            ]
//   }
// }
// object { MTVC_TexturaMetal() }
// otros modificadores ( se indican valores default)
// #declare MTVC_POV3_1                 = off;
//      "on" desactiva procesos y algoritmos POVRAY 3.6, 
//      para mantener compatibilidad con POVRAY 3.1.
//      en algunas versiones de POVRay se activa automticamente. 
// #declare MTVC_FactorLargoEstimadoV   = 1.0;  
//      en caso de que los decorados superiores, adyacentes al vidrio
//      resulten muy largos o muy pequeos, puede usarse
//      este factor para intentar remediar el problema.
// #declare MTVC_FactorDecorado         = 0.98;
//      si el decorado lateral queda muy adentro o muy afuera,
//      podria utilizarse este modificador para "hacerlo" calzar.
//
  
#include "colors.inc"  

#ifndef ( MTVC_POV3_1 )
  #if ( version < 3.2 )
    #declare MTVC_POV3_1        = on;
  #else
    #declare MTVC_POV3_1        = off;
  #end
#end


#include "textures.inc"

#if ( MTVC_POV3_1 = off ) 
  #include "stones2.inc"
#end

#declare MTVC_L         = 0.001;

#declare MTVC_TexturaMetal       = texture       {
  #if ( MTVC_POV3_1 )
    pigment       { White }
  #else
    T_Stone31//pigment       { White }
  #end
}

#ifndef ( MTVC_TexturaMetalDefault )
  #declare MTVC_TexturaMetalDefault            = texture       {
    MTVC_TexturaMetal
  }
#end

#declare MTVC_MaterialVidrio     = material      {
  #if ( MTVC_POV3_1 )
    texture     {
      pigment  { rgbt < 0.8,1,1, 0.7 > }
    }
  #else
    M_Glass
  #end
}

#ifndef ( MTVC_MaterialVidrioDefault )     
  #declare MTVC_MaterialVidrioDefault          = material      {
    MTVC_MaterialVidrio
  }
#end

#declare MTVC_RadioPrincipal            = 325*MTVC_L;

#declare MTVC_RadioMedio                = 186/2*MTVC_L;

#macro MTVC_PuntoRadialXZ ( Radio, LongArc )  
  < Radio*cos( LongArc/Radio ), 0.0, Radio*sin( LongArc/Radio ) >
#end

#macro MTVC_Cor ( MyPunto )  
  < MyPunto.x*cos( MyPunto.z/MyPunto.x ), MyPunto.y, 
    MyPunto.x*sin( MyPunto.z/MyPunto.x ) >
#end  

#macro MTVC_MesaTerrazaVidrioCircular 
  ( RadioPrincipal, RadioMedio, FactorGrueso, TexturaMetal, MaterialVidrio )
  #local L                      = 0.001;
  #local DiametroSweep          = FactorGrueso*9.3*L;
  #local DiametroSweepFino      = FactorGrueso*4.5*L;
  #local AlturaCirculoMedio     = 232*L;
  #local AlturaCirculoAlto      = 454*L;
  #local AltoAnilloSuperior     = FactorGrueso*16*L;
  #local AlturaAnilloSuperior   = 514*L;
  #local GruesoAnilloSuperior   = FactorGrueso*4*L;
  
  // diseo de patas   
  #local FactorBasePata         = 1.0;
  #local SeparacionMedioPata    = 50*L;
  #local SeparacionAltoPata     = 68*L;
  #local BasePata               = MTVC_PuntoRadialXZ
           ( FactorBasePata*RadioPrincipal, DiametroSweep/2 )+
           DiametroSweep/2*y;
  #local MedioPata              = MTVC_PuntoRadialXZ
           ( RadioMedio+DiametroSweep, SeparacionMedioPata )+
           AlturaCirculoMedio*y;
  #local AltoPata               = MTVC_PuntoRadialXZ
           ( RadioPrincipal, SeparacionAltoPata )+
           AlturaCirculoAlto*y;
  #local FrenteDecoradoV        = 71*L;         
  #local GruesoDecoradoV        = FactorGrueso*6.5*L;   
  #local LargoDecoradoV         = RadioPrincipal/2; 
  #local AltoSeguridadVidrio    = FactorGrueso*21*L;
  #local GruesoVidrio           = AltoSeguridadVidrio-AltoAnilloSuperior;
  #local FrenteSeguridadVidrio  = 18*L;  
  #ifndef ( MTVC_FactorLargoEstimadoV )
    #declare MTVC_FactorLargoEstimadoV  = 1.0;
  #end
  #local LargoEstimadoV         =
                ( RadioPrincipal-FrenteDecoradoV/sqrt(2)-GruesoDecoradoV-
                        GruesoAnilloSuperior/2 )*
               cos( ( FrenteDecoradoV+GruesoDecoradoV )/RadioPrincipal )
               *MTVC_FactorLargoEstimadoV;
  #local GruesoDecoradoC        = FactorGrueso*4.5*L;
  #local AltoDecoradoC          = 
                AlturaAnilloSuperior-AlturaCirculoAlto+DiametroSweep/2-
                GruesoDecoradoC;                   
  #local FrenteDecoradoC        = 39*L;  
  #local LargoDecoradoC         = 138*L; 
  #local PuntoDecorado          = 
                < RadioPrincipal, AltoDecoradoC, FrenteDecoradoC >;
  #ifndef ( MTVC_FactorDecorado )
    #declare MTVC_FactorDecorado        = 0.98;
  #end

  #if ( MTVC_POV3_1 )
    #local CurvaPata    = union {
      cylinder  {
        BasePata        , MedioPata     , DiametroSweep/2
      }
      cylinder  {
        MedioPata       , AltoPata      , DiametroSweep/2
      }
    }
  #else
    #local CurvaPata              = sphere_sweep  {
      cubic_spline
      5,
      2*BasePata+MedioPata*<2,-1,-1>       , DiametroSweep/2       ,
      BasePata            , DiametroSweep/2       ,
      MedioPata           , DiametroSweep/2       ,
      AltoPata            , DiametroSweep/2       ,
      2*AltoPata          , DiametroSweep/2
    }
  #end
  #local DecoradoBaseVidrio      = union         {
    difference          {
      torus             {
        FrenteDecoradoV/2       , GruesoDecoradoV       
      }
      box               {
        <0,-1,-1>        , <1,1,0>
      }
    }
    difference          {
      torus             {
        FrenteDecoradoV/5       , GruesoDecoradoV       
      }
      plane             {
        -z      , 0
      }
      translate         3*FrenteDecoradoV/10*x
    }
    cylinder            {
      0,        ( LargoDecoradoV )*x  ,
      GruesoDecoradoV
      translate         -FrenteDecoradoV/2*z  
    }
  }
  #local DecoradoVidrio        = union {
    object      {
      DecoradoBaseVidrio
      translate < ( FrenteDecoradoV+GruesoDecoradoV )/2, 0,
                    -( FrenteDecoradoV+GruesoDecoradoV )/2 >
    }
    object      {
      DecoradoBaseVidrio 
      scale     <-1,+1,-1>
      translate < LargoEstimadoV, 0,
                  -3*FrenteDecoradoV/2-GruesoDecoradoV/2 >
    }
    translate   
      ( +AlturaAnilloSuperior+AltoAnilloSuperior-GruesoDecoradoV/2 )*y                       
  }
  #local DecoradoBaseCirculo     = union         {
    difference          {
      torus             {
        FrenteDecoradoC/2       , GruesoDecoradoC
        rotate          90*x       
      }
      box               {
        <0,0,-1>        , <1,-1,1>
      }
    } 
    difference          {
      torus             {
        FrenteDecoradoC/4       , GruesoDecoradoC
        rotate          90*x       
      }
      plane             {
        -y      , 0
      }
      translate         2*FrenteDecoradoC/8*x
    }
    cylinder            {
      0,        ( LargoDecoradoC-FrenteDecoradoC )/2*x  ,
      GruesoDecoradoC
      translate         -FrenteDecoradoC/2*y  
    } 
    translate   -FrenteDecoradoC*x
  }
  #local DecoradoCirculo         = union {
    object      {
      DecoradoBaseCirculo 
    }
    object      {
      DecoradoBaseCirculo 
      scale     <-1,+1,+1>
    }
    scale       1.2*y+x+z
    translate   (AlturaCirculoAlto+DiametroSweep/2-1.0*GruesoDecoradoC/2
                +AltoDecoradoC/2)*y
                        -MTVC_FactorDecorado*RadioPrincipal*z
                       
  }
  union         {  
    torus       {
      RadioMedio        , DiametroSweep/2
      translate         AlturaCirculoMedio*y        
    }            
    torus       {
      RadioPrincipal    , DiametroSweep/2
      translate         AlturaCirculoAlto*y
    }
    difference  {
      cylinder  {
        0       , AltoAnilloSuperior*y  , 
        RadioPrincipal+GruesoAnilloSuperior/2
      }
      cylinder  {
        -0.01*AltoAnilloSuperior*y      , 1.01*AltoAnilloSuperior*y
        RadioPrincipal-GruesoAnilloSuperior/2
      } 
      translate         AlturaAnilloSuperior*y
    }  
    #local C            = 0;
    #while ( C < 4 )
      object            {
        CurvaPata
        rotate          90*C*y
      }
      object            {
        CurvaPata
        scale           <1,1,-1>
        rotate          90*C*y
      }
      #local C          = C+1;
    #end  
    #local C            = 0;
    #while ( C < 8 )          
      object            {
        DecoradoCirculo
        rotate          -45*C*y
      }
      #local C          = C+1;
    #end   
    #local C            = 0;
    #while ( C < 4 )
      object            {
        DecoradoVidrio
        rotate          90*C*y  
      }
      box               {
        < RadioPrincipal, AlturaAnilloSuperior,
          -FrenteSeguridadVidrio/2 >,
        < RadioPrincipal+GruesoAnilloSuperior,
          AlturaAnilloSuperior+AltoSeguridadVidrio,
          +FrenteSeguridadVidrio/2 >
        rotate          90*C*y
      }
      #local C          = C+1;
    #end      
    cylinder    {
      0         , GruesoVidrio*y        ,  
      RadioPrincipal
      translate        
        ( AlturaAnilloSuperior+AltoSeguridadVidrio-GruesoVidrio )*y
      material  {
        MaterialVidrio
      }
    }
    texture     {
      TexturaMetal
    }
  }
#end

#macro MTVC_MesaTerraza ()
  object        {
    MTVC_MesaTerrazaVidrioCircular
      ( MTVC_RadioPrincipal, MTVC_RadioMedio , 1.0,
        MTVC_TexturaMetalDefault, MTVC_MaterialVidrioDefault )
  }
#end