// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com
// Apdo. 964-1250 Escaz, San Jos, Costa Rica 
// dedicado a Jess y Mara...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 31 enero 2011
//                
// Mis dibujos estn en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY 
//
// "TrasteroSMB.inc" dibuja un Trastero de sala
// Sus dimensiones son ( sin deformaciones adicionales ):
// Alto                 = 1392*L
// AnchoFrontal         = 894*L
// AnchoLateral         = 307*L
//   mas un ligero exceso hacia adelante
//   por la agarradera de 10*L
//   mas una capita delgada de madera atras de 2*L
// Su ubicacion primaria es con la base trasera centrada en el origen
// ( sin incluir la capita delgada )
// su Ancho Frontal apuntando hacia z-
// Las dimensiones de la macro default variaran un poco
// por pseudo-aleatorios desplazamientos de gaveta y puerta.
//
// Para ponerlo en pantalla simplemente utilice la macro default
// #include "TSMB.inc"
// object { TSMB_Trastero1() }
// Otra alternativa disponible es utilizar la macro:
// object { TSMB_TrasteroSalaMaderaBarnizada
//  ( TexturaBase, TexturaMetal, TexturaBarata,
//        Gaveta1, Gaveta2, Gaveta3, Puerta1, Puerta2, Puerta3, Central )
//      TexturaBase     : Textura principal del objeto
//      TexturaMetal    : Textura de las agarraderas metalicas
//      TexturaBarata   : El interior de las gavetas, la parte
//              posterior del trastero, utiliza madera de inferior calidad
//      Gaveta1         : fraccion de 0 a 1 de apertura de Gaveta
//                              Lateral Izquierda
//      Gaveta2         : fraccion de 0 a 1 de apertura de Gaveta 
//                              Central Inferior
//      Gaveta3         : fraccion de 0 a 1 de apertura de Gaveta
//                              Lateral Derecha
//      Puerta1         : Angulo grados de apertura puerta compartimento
//                              Inferior Lateral Izquierdo
//      Puerta2         : Angulo grados de apertura puerta compartimento
//                              Inferior Central
//      Puerta3         : Angulo grados de apertura puerta compartimento
//                              Inferior Lateral Derecho
//      Central         : fraccion de 0 a 1 de apertura de Gaveta
//                              Superior Central
// Un valor 1.0 de apertura representa como un 70% del recorrido
//
// Para accesar a las siguientes texturas basicas que
// no deberian modificarse, las variables son
// #declare TSMB_TexturaBaseTrastero
// #declare TSMB_TexturaMaderaBarato 
// #declare TSMB_TexturaMetalBarato
// Si usted desea modificar la textura default, utilice 
// las siguientes variables tipo "#ifndef"
// #declare TSMB_TexturaBaseTrasteroDefault
// #declare TSMB_TexturaMaderaBaratoDefault
// #declare TSMB_TexturaMetalBaratoDefault 
// Para activar compatibilidad con POVRay 3.1.
//    (innecesario pero se recomienda)
// #declare TSMB_POV3_1         = on;
// Por ejemplo 
// #include "TSMB.inc"
// #declare CustomTextura               = texture       {
//   pigment            { Cyan }
// }
// #declare TSMB_TexturaBaseTrasteroDefault = texture       {
//   average
//   texture_map   {
//      [ 0.5 TSMB_TexturaBaseTrastero ]
//      [ 0.5 CustomTextura            ]
//   }
// }
// object { TSMB_Trastero1() }
// Donde CustomTextura seria una textura de su propia creacion 
// Cada llamado de la macro genera las siguientes variables
// TSMB_AlturaGaveta[3][2]
//           [0][0] : Esquina interior Gaveta Lateral Izquierda
//                      apunta hacia <-1,-1,-1>
//           [0][1] : Esquina interior Gaveta Lateral Izquierda
//                      apunta hacia <+1,+1,+1>
//           [1][0] : Esquina interior Gaveta Central          
//                      apunta hacia <-1,-1,-1>
//           [1][1] : Esquina interior Gaveta Central           
//                      apunta hacia <+1,+1,+1>
//           [2][0] : Esquina interior Gaveta Lateral Derecha  
//                      apunta hacia <-1,-1,-1>
//           [2][1] : Esquina interior Gaveta Lateral Derecha   
//                      apunta hacia <+1,+1,+1 >
// Las direcciones se refieren al sistema original de creacion
// de la macro, tal como se explica arriba, include fraccion de apertura
// TSMB_AlturaNivel1    : altura y objetos del nivel sobre las gavetas
// TSMB_AlturaNivel2    : altura y objetos nivel gavetilla central
// TSMB_AlturaNivel3    : altura y objetos nivel tercero
// TSMB_AlturaNivel4    : altura y de la zona superior del trastero
//  

#include "colors.inc"
#include "woods.inc"
#include "metals.inc"
                       
#declare TSMB_TexturaBaseTrastero       = texture       {
  T_Wood17
}

#declare TSMB_TexturaMaderaBarata       = texture       {
  T_Wood9
  scale         100
  translate     20
  rotate        300
}

#local MyTextureSilver                  = texture       {
  pigment       { Gold*0.5 }
  normal        { spiral2 20, 0.1 scale 0.01 }
}

#declare TSMB_TexturaMetalBarato        = texture       {
  average
  texture_map   {
    [ 0.7 MyTextureSilver ]
    [ 0.3 T_Brass_3E      ]
  }
}

#ifndef ( TSMB_TexturaMetalBaratoDefault )
  #declare TSMB_TexturaMetalBaratoDefault       = texture       {
    TSMB_TexturaMetalBarato
  }
#end

#ifndef ( TSMB_TexturaBaseTrasteroDefault )
  #declare TSMB_TexturaBaseTrasteroDefault      = texture       {
    TSMB_TexturaBaseTrastero
  }
#end

#ifndef ( TSMB_TexturaMaderaBarataDefault )
  #declare TSMB_TexturaMaderaBarataDefault      = texture       {
    TSMB_TexturaMaderaBarata
  }
#end

#macro TSMB_TrasteroSalaMaderaBarnizada
  ( TexturaBase, TexturaMetal, TexturaBarata,
        Gaveta1, Gaveta2, Gaveta3, Puerta1, Puerta2, Puerta3, Central )
  #local L                      = 0.001;
  #local AltoSuelo              = 57*L;
  #local AltoBase               = 160*L;
  #local FrenteBase             = 930*L;
  #local ProfundidadBase        = 336*L;
  #local AltoGavetero           = 514*L;
  #local FrenteGavetero         = 894*L;
  #local ProfundidadGavetero    = 307*L;
  #local MaderaGruesa           = 16*L;
  #local MaderaMedia            = 9*L;
  #local MaderaFina             = 2*L;
  #local MaderaPiso             = 12.5*L;
  #local VacioPiso1             = 209*L;
  #local VacioPiso2             = 202*L;
  #local ProfundidadPiso        = 244*L;
  #local MaderaBase             = MaderaGruesa;
  #local AltoPrincipal          = 1165*L;
  #local AltoTrastero           = AltoPrincipal-AltoGavetero;
  #local FrenteTrastero         = FrenteGavetero;
  #local AltoSuperior           = 67*L;
  #local FrenteSuperior         = 873*L;
  #local ProfundidadSuperior    = 300*L;
  #local CorrimientoSuperiorZ   = 5*L;
  #local ProfundidadTrastero    = 307*L;
  #local DiametroPataMayor      = 35*L;
  #local DiametroPataMenor      = 21*L;
  #local RadioTorusPata         = ( DiametroPataMayor-DiametroPataMenor )/2;
  #local DiametroPataMedio      = ( DiametroPataMayor+DiametroPataMenor )/2;
  #local GruesoGaveta           = 8.5*L;
  // Base Piso
  #local PataBasica             = union                 {
    difference          {
      cylinder          {
        0       , AltoBase*y            ,
        DiametroPataMedio/2
      }
      torus             {
        DiametroPataMedio/2     , 0.5*RadioTorusPata
        scale   <1,1,1>
        translate       AltoSuelo/2*y
      }
    }
    torus               {
      DiametroPataMedio/2       , RadioTorusPata
      scale     <1,1,1>
      translate 1*RadioTorusPata*y
    }
    torus               {
      DiametroPataMedio/2       , RadioTorusPata
      scale     <1,1,1>
      translate AltoSuelo*y-1*RadioTorusPata*y
    }
  }
  #local DecoradoCurvo          = difference            {
    #local FraccionDecorado     = 0.8;
    union       {
      box         {
        0         , 1
      }
      cylinder    {
        0.5*y+FraccionDecorado/2*x,
        0.5*y+FraccionDecorado/2*x+z,
        sqrt(2)*FraccionDecorado/2
      }
    }
    cylinder    {

      -( 1-FraccionDecorado )/2*x-0.1*z,
      -( 1-FraccionDecorado )/2*x+1.1*z,
      ( 1-FraccionDecorado )/2*sqrt(2)
    }
    cylinder    {
      -( 1-FraccionDecorado )/2*x+y-0.1*z,
      -( 1-FraccionDecorado )/2*x+y+1.1*z,
      ( 1-FraccionDecorado )/2*sqrt(2)
    }
    bounded_by  {
      box       {
        -x      , 1
      }
    }
    translate   -( ( 1-FraccionDecorado )/2*( sqrt(2)-1 ) )*x
  }

  #local DecoradoPuerta         = difference            {
    #local RadioEsferaPuerta    = 0.025;
    union       {
    sphere      {
      ( 1-RadioEsferaPuerta )*y, RadioEsferaPuerta*1.5
    }
    torus       {
      RadioEsferaPuerta , RadioEsferaPuerta
      translate 3*RadioEsferaPuerta*y
    }
    sphere      {
      0, 1
      scale
        < 2*RadioEsferaPuerta,
          1.1*( 0.25-3*RadioEsferaPuerta ), 2*RadioEsferaPuerta >
      translate 0.5*y+2*RadioEsferaPuerta*y+( 0.5-6*RadioEsferaPuerta )/2*y
    }
    torus       {
      RadioEsferaPuerta , RadioEsferaPuerta
      translate y-3*RadioEsferaPuerta*y
    }
    torus       {
      RadioEsferaPuerta , RadioEsferaPuerta
      translate 0.5*y+RadioEsferaPuerta*y
    }
    sphere      {
      0, 1
      scale
        < 2*RadioEsferaPuerta,
          1.1*( 0.25-3*RadioEsferaPuerta ), 2*RadioEsferaPuerta >
      translate 0.5*y-2*RadioEsferaPuerta*y-( 0.5-6*RadioEsferaPuerta )/2*y
    }
    torus       {
      RadioEsferaPuerta , RadioEsferaPuerta
      translate 0.5*y-RadioEsferaPuerta*y
    }
    sphere      {
      ( RadioEsferaPuerta )*y, RadioEsferaPuerta*1.5
    }
    }
    plane       {
      -z , 0
    }
  }

  #local BalconcitoPuerta       = difference            {
    #local RadioEsferaPuerta    = 0.025;
    union       {
      sphere      {
        ( 1-RadioEsferaPuerta )*y, RadioEsferaPuerta*1.5
      }
      torus       {
        RadioEsferaPuerta , RadioEsferaPuerta
        translate 3*RadioEsferaPuerta*y
      }
      cylinder    {
        ( 1-3*RadioEsferaPuerta )*y, ( 1-7*RadioEsferaPuerta )*y
        RadioEsferaPuerta
      }
      difference        {
        sphere          {
          0     , 1
          scale         < 3*RadioEsferaPuerta,
                        RadioEsferaPuerta, RadioEsferaPuerta >
        }
        torus           {
          1.5*RadioEsferaPuerta   , RadioEsferaPuerta
          rotate        90*z
        }
        translate     ( 1-5*RadioEsferaPuerta )*y+
                        3*RadioEsferaPuerta*x
      }
      torus       {
        RadioEsferaPuerta , RadioEsferaPuerta
        translate ( 1-7*RadioEsferaPuerta )*y
      }
      sphere      {
        0, 1
        scale
          < 2*RadioEsferaPuerta,
            1.1*( 0.25-5.5*RadioEsferaPuerta ), 2*RadioEsferaPuerta >
        translate 0.5*y+4*RadioEsferaPuerta*y+( 0.5-11*RadioEsferaPuerta )/2*y
      }
      torus       {
        RadioEsferaPuerta , RadioEsferaPuerta
        translate y-3*RadioEsferaPuerta*y
      }
      torus       {
        RadioEsferaPuerta , RadioEsferaPuerta
        translate 0.5*y+3*RadioEsferaPuerta*y
      }
      sphere      {
        0, 1
        scale
          < 2*RadioEsferaPuerta,
            1.1*( 0.25-5.5*RadioEsferaPuerta ), 2*RadioEsferaPuerta >
        translate 0.5*y-4*RadioEsferaPuerta*y-( 0.5-11*RadioEsferaPuerta )/2*y
      }
      cylinder    {
        ( 3*RadioEsferaPuerta )*y, ( 7*RadioEsferaPuerta )*y
        RadioEsferaPuerta
      }
      difference        {
        sphere          {
          0     , 1
          scale         < 3*RadioEsferaPuerta,
                        RadioEsferaPuerta, RadioEsferaPuerta >
        }
        torus           {
          1.5*RadioEsferaPuerta   , RadioEsferaPuerta
          rotate        90*z
        }
        translate     0.5*y+
                        3*RadioEsferaPuerta*x
      }
      torus       {
        RadioEsferaPuerta , RadioEsferaPuerta
        translate 7*RadioEsferaPuerta*y
      }
      cylinder    {
        0.5*y-3*RadioEsferaPuerta*y,
        0.5*y+3* RadioEsferaPuerta*y,
        RadioEsferaPuerta
      }
      difference        {
        sphere          {
          0     , 1
          scale         < 3*RadioEsferaPuerta,
                        RadioEsferaPuerta, RadioEsferaPuerta >
        }
        torus           {
          1.5*RadioEsferaPuerta   , RadioEsferaPuerta
          rotate        90*z
        }
        translate     5*RadioEsferaPuerta*y+
                        3*RadioEsferaPuerta*x
      }
      torus       {
        RadioEsferaPuerta , RadioEsferaPuerta
        translate 0.5*y-3*RadioEsferaPuerta*y
      }
      sphere      {
        ( RadioEsferaPuerta )*y, RadioEsferaPuerta*1.5
      }
    }
    plane       {
      -z , 0
    }
  }

  #local Agarradera     = union {
    #local RadioMayor   = 12.5*L;
    #local RadioMenor   = 3.5*L;
    #local ProfundidadMenor     = 9*L;
    #local ProfundidadMayor     = 1*L;
    cylinder            {
      +ProfundidadMenor/4*z , -ProfundidadMenor*z
      RadioMenor
    }
    cylinder            {
      -ProfundidadMenor*z,
      -( ProfundidadMayor+ProfundidadMenor )*z,
      RadioMayor
    }
    texture     { TexturaMetal }
  }
  
  union       {
    union       {
      difference        {
        box       {
          < -FrenteBase/2 , AltoSuelo           , -ProfundidadBase      >,
          < +FrenteBase/2 , AltoBase            , 0                     >
        }
        
        box     {
          < -FrenteBase/2 , AltoSuelo   , -ProfundidadBase      >
                +MaderaGruesa*<+1,-1,+1>,
          < +FrenteBase/2 , AltoBase    , 0                     >
                +MaderaGruesa*<-1,-1,-1>
        }
        object  {
          DecoradoCurvo
          rotate        -90*z
          translate     y
          scale         < FrenteBase/4*0.9, 2*AltoSuelo   , 1     >
          translate     +FrenteBase/4*0.9*x-2*ProfundidadBase*z-AltoSuelo*y
        }
        object  {
          DecoradoCurvo
          rotate        -90*z
          translate     y
          scale         < FrenteBase/4*0.9, 2*AltoSuelo   , 1     >
          translate     -FrenteBase/2*0.9*x-2*ProfundidadBase*z-AltoSuelo*y
        }
        object  {
          DecoradoCurvo
          rotate        -90*z
          translate     y
          scale         < FrenteBase/4*0.9, 2*AltoSuelo   , 1     >
          translate     -FrenteBase/4*0.9*x-2*ProfundidadBase*z-AltoSuelo*y
        }
        object  {
          DecoradoCurvo
          rotate        -90*z
          translate     y
          scale         < FrenteBase/4*0.9, 2*AltoSuelo   , 1     >
          translate     -2*ProfundidadBase*z-AltoSuelo*y
        }
          object  {
            DecoradoCurvo
            rotate        -90*z
            translate     y
            scale         < ProfundidadBase/2*0.7, 2*AltoSuelo   , 1     >
            translate     -0.5*z-AltoSuelo*y
            rotate        90*y
            translate     -ProfundidadBase/2*z
          }
          object  {
            DecoradoCurvo
            rotate        -90*z
            translate     y
            scale         <-ProfundidadBase/2*0.7, 2*AltoSuelo   , 1     >
            translate     -0.5*z-AltoSuelo*y
            rotate        90*y
            translate     -ProfundidadBase/2*z
          }
      }
      // Patas
      #local CentroPataDelantero        =
        < -FrenteBase/2+MaderaGruesa+DiametroPataMayor/2        , AltoSuelo,
          -ProfundidadBase+MaderaGruesa+DiametroPataMayor/2     >;
      #local CentroPataTrasero          =
        < -FrenteBase/2+MaderaGruesa+DiametroPataMayor/2        , AltoSuelo,
          -MaderaGruesa-DiametroPataMayor/2     >;
      object            {
        PataBasica
        translate       CentroPataDelantero*<+1,+0,+1>
      }
      object            {
        PataBasica
        translate       CentroPataDelantero*<-1,+0,+1>
      }
      object            {
        PataBasica
        translate       CentroPataTrasero*<-1,+0,+1>
      }
      object            {
        PataBasica
        translate       CentroPataTrasero*<+1,+0,+1>
      }
    }
    // Base Gavetas Inferiores
    union       {
      difference  {
        union   {
          box       {
            < -FrenteGavetero/2   , 0             ,
                -ProfundidadGavetero    >,
            < +FrenteGavetero/2   , AltoGavetero  ,
                +0                      >
          }
          box           {
            < -FrenteGavetero/2 , 0             , 0             >,
            < +FrenteGavetero/2 , AltoPrincipal  , +MaderaFina   >
            texture     { TexturaBarata }
          }
        }
        box       {
          < -FrenteGavetero/2   , 0             ,
                -ProfundidadGavetero    >+MaderaGruesa*<+1,-1,-1>,
          < +FrenteGavetero/2   , AltoGavetero  ,
                +0                      >+MaderaGruesa*<-1,-1,-1>
        }
      }
      #local AltoHuecoGaveta            = 82*L;
      #local AltoSeparadorVertical      = 117*L-AltoHuecoGaveta;
      #local HuecoPuertaInferior        =
                AltoGavetero-MaderaGruesa
                -AltoHuecoGaveta-AltoSeparadorVertical;
      #local VacioGavetaLateral         = 271*L;
      #local VacioGavetaCentral         = 257*L;
      #local FrenteGavetaLateral        = 297*L;
      #local FrenteGavetaCentral        = 279*L;
      box       {
        < -FrenteGavetero/2     , HuecoPuertaInferior   ,
                -ProfundidadGavetero    >,
        < +FrenteGavetero/2     , HuecoPuertaInferior+AltoSeparadorVertical,
                -ProfundidadGavetero+MaderaGruesa       >
      }
      box       {
        < -FrenteGavetero/2+MaderaGruesa+VacioGavetaLateral     ,
          0     , -ProfundidadGavetero  >,
        < -VacioGavetaCentral/2 , AltoGavetero                  ,
          -ProfundidadGavetero+MaderaGruesa     >
      }
      box       {
        < -FrenteGavetero/2+MaderaGruesa+VacioGavetaLateral     ,
          0     , -ProfundidadGavetero  >*<-1,+1,+1>,
        < -VacioGavetaCentral/2 , AltoGavetero                  ,
          -ProfundidadGavetero+MaderaGruesa     >*<-1,+1,+1>
      }
      // Soportes baratos internos
      #local GruesoInterior             = 11*L;
      #local FrenteReglaInterior        = 46*L;
      #local ExcesoInterior             =
                ( FrenteReglaInterior-
                  FrenteGavetero/2+MaderaGruesa+3*VacioGavetaLateral/2 )/2;
      union     {
        box       {
          < -FrenteGavetero/2+MaderaGruesa+VacioGavetaLateral     ,
            AltoGavetero-AltoHuecoGaveta-MaderaGruesa     ,
            -ProfundidadGavetero+GruesoInterior/2  >,
          < -VacioGavetaCentral/2 ,
            AltoGavetero+GruesoInterior-AltoHuecoGaveta-MaderaGruesa   ,
            -GruesoInterior     >
        }
        box       {
          < -FrenteGavetero/2+MaderaGruesa+VacioGavetaLateral     ,
            AltoGavetero-AltoHuecoGaveta-MaderaGruesa     ,
            -ProfundidadGavetero+GruesoInterior/2  >*<-1,+1,+1>,
          < -VacioGavetaCentral/2 ,
            AltoGavetero+GruesoInterior-AltoHuecoGaveta-MaderaGruesa   ,
            -GruesoInterior     >*<-1,+1,+1>
        }
        box       {
          < -FrenteGavetero/2+MaderaGruesa+VacioGavetaLateral
            -ExcesoInterior,
            AltoGavetero-AltoHuecoGaveta-MaderaGruesa     ,
            -ProfundidadGavetero+GruesoInterior  >,
          < -VacioGavetaCentral/2+ExcesoInterior ,
            AltoGavetero-GruesoInterior-AltoHuecoGaveta-MaderaGruesa   ,
            -GruesoInterior     >
        }
        box       {
          < -FrenteGavetero/2+MaderaGruesa+VacioGavetaLateral
            -ExcesoInterior,
            AltoGavetero-AltoHuecoGaveta-MaderaGruesa     ,
            -ProfundidadGavetero+GruesoInterior  >*<-1,+1,+1>,
          < -VacioGavetaCentral/2+ExcesoInterior ,
            AltoGavetero-GruesoInterior-AltoHuecoGaveta-MaderaGruesa   ,
            -GruesoInterior     >*<-1,+1,+1>
        }
        texture         {
          TexturaBarata
        }
      }
      translate ( AltoBase )*y
    }
    // Gavetas
    #macro TSMB_Gaveta ( FrenteInterior, Frente )
       #local AltoFrenteGaveta          = 94*L;
       #local MargenPosteriorGaveta     = 7*L;
       union    {
         union          {
           box          {
             < -FrenteInterior/2        , 0     , -GruesoGaveta >,
             < +FrenteInterior/2        ,
               AltoHuecoGaveta-MargenPosteriorGaveta, 0     >
           }
           box          {
             < -FrenteInterior/2        , 0     ,
               -ProfundidadGavetero+MaderaGruesa        >,
             < -FrenteInterior/2+GruesoGaveta   , AltoHuecoGaveta, 0    >
           }
           box          {
             < -FrenteInterior/2        , 0     ,
               -ProfundidadGavetero+MaderaGruesa        >*<-1,1,1>,
             < -FrenteInterior/2+GruesoGaveta   , AltoHuecoGaveta, 0    >*
                        <-1,1,1>
           }
           box          {
             < -FrenteInterior/2        , 0     ,
               -ProfundidadGavetero+MaderaGruesa        >,
             < +FrenteInterior/2        , MaderaFina, 0    >
           }
           box          {
             < -Frente/2        , AltoHuecoGaveta-AltoFrenteGaveta      ,
               -ProfundidadGavetero+MaderaGruesa-GruesoGaveta   >,
             < +Frente/2        , AltoHuecoGaveta                       ,
               -ProfundidadGavetero+MaderaGruesa                >
             texture    {
               TexturaBase
             }
           }
           object       {
             Agarradera
             translate  -ProfundidadGavetero*z+MaderaGruesa*z
                        +0.60*AltoHuecoGaveta*y
           }
           texture      {
             TexturaBarata
           }
         }
       }
    #end

    #macro TSMB_PuertaInferior ( Frente )
      #local AltoPuertaInferior = 394*L;
      #local FraccionDecorado   = 0.7;
      union     {
        box             {
          0     ,
          < Frente      , AltoPuertaInferior    , -GruesoGaveta >
        }
        object          {
          DecoradoPuerta
          scale         AltoPuertaInferior*FraccionDecorado
          translate     ( 1-FraccionDecorado )*AltoPuertaInferior/2*y
                +1*Frente/3*x-GruesoGaveta*z
        }
        object          {
          DecoradoPuerta
          scale         AltoPuertaInferior*FraccionDecorado
          translate     ( 1-FraccionDecorado )*AltoPuertaInferior/2*y
                +2*Frente/3*x-GruesoGaveta*z
        }
        object       {
          Agarradera
          translate  -GruesoGaveta*z+5*Frente/6*x
                     +0.50*AltoPuertaInferior*y
        }
        texture         {
          TexturaBase
          scale         2
          translate     2
          rotate        135*x
        }
      }
    #end
    // Base Trastero Superior
    #local FraccionApertura             = 0.75;
    object      {
      TSMB_Gaveta( VacioGavetaLateral, FrenteGavetaLateral )
      translate    ( AltoBase+AltoGavetero-AltoHuecoGaveta-MaderaGruesa )*y
      translate    ( -FrenteGavetero/2+MaderaGruesa+VacioGavetaLateral/2 )*x
      translate    -MaderaGruesa*z
                -FraccionApertura*Gaveta1*ProfundidadGavetero*z
    }
    object      {
      TSMB_Gaveta( VacioGavetaCentral, FrenteGavetaCentral )
      translate    ( AltoBase+AltoGavetero-AltoHuecoGaveta-MaderaGruesa )*y
      translate    -MaderaGruesa*z
                -FraccionApertura*Gaveta2*ProfundidadGavetero*z
    }
    object      {
      TSMB_Gaveta( VacioGavetaLateral, FrenteGavetaLateral )
      translate    ( AltoBase+AltoGavetero-AltoHuecoGaveta-MaderaGruesa )*y
      translate    -( -FrenteGavetero/2+MaderaGruesa+VacioGavetaLateral/2 )*x
      translate    -MaderaGruesa*z
                -FraccionApertura*Gaveta3*ProfundidadGavetero*z
    }
    #declare TSMB_AlturaGaveta          = array [3][2]
        { { < -FrenteGavetero/2+MaderaGruesa+GruesoGaveta,
              AltoBase+AltoGavetero-AltoHuecoGaveta
              +GruesoGaveta-MaderaGruesa,
              -ProfundidadGavetero >
              -FraccionApertura*Gaveta1*ProfundidadGavetero*z,
            < -FrenteGavetero/2+MaderaGruesa
              +VacioGavetaLateral-GruesoGaveta,
              AltoBase+AltoGavetero-MaderaGruesa,
              -MaderaGruesa-GruesoGaveta >
              -FraccionApertura*Gaveta1*ProfundidadGavetero*z }
          { < -VacioGavetaCentral/2+GruesoGaveta,
              AltoBase+AltoGavetero-AltoHuecoGaveta
              +GruesoGaveta-MaderaGruesa,
              -ProfundidadGavetero >
              -FraccionApertura*Gaveta2*ProfundidadGavetero*z,
            < +VacioGavetaCentral/2-GruesoGaveta,
              AltoBase+AltoGavetero-MaderaGruesa,
              -MaderaGruesa-GruesoGaveta >
              -FraccionApertura*Gaveta2*ProfundidadGavetero*z }
          { < +FrenteGavetero/2-MaderaGruesa
              -VacioGavetaLateral+GruesoGaveta,
              AltoBase+AltoGavetero-AltoHuecoGaveta
              +GruesoGaveta-MaderaGruesa,
              -ProfundidadGavetero >
              -FraccionApertura*Gaveta3*ProfundidadGavetero*z,
            < +FrenteGavetero/2-MaderaGruesa-GruesoGaveta,
              AltoBase+AltoGavetero-MaderaGruesa,
              -MaderaGruesa-GruesoGaveta >
              -FraccionApertura*Gaveta3*ProfundidadGavetero*z }
            }   
    #declare TSMB_AlturaNivel1  = AltoBase+AltoGavetero;
    #declare TSMB_AlturaNivel2  = TSMB_AlturaNivel1+
                VacioPiso1+MaderaPiso;
    #declare TSMB_AlturaNivel3  = TSMB_AlturaNivel2+
                VacioPiso2+MaderaPiso;
    #declare TSMB_AlturaNivel4  = AltoBase+AltoGavetero+AltoTrastero;
    object      {
      TSMB_PuertaInferior( FrenteGavetaLateral )
      rotate    Puerta1*y
      translate +AltoBase*y-ProfundidadGavetero*z+
                ( -FrenteGavetero/2+MaderaGruesa+VacioGavetaLateral/2
                        -FrenteGavetaLateral/2 )*x
    }
    object      {
      TSMB_PuertaInferior( FrenteGavetaCentral )
      rotate    Puerta2*y
      translate +AltoBase*y-ProfundidadGavetero*z
                -FrenteGavetaCentral/2*x
    }
    object      {
      TSMB_PuertaInferior( FrenteGavetaLateral )
      scale     <-1,+1,+1>
      rotate    -Puerta3*y
      translate +AltoBase*y-ProfundidadGavetero*z-
                ( -FrenteGavetero/2+MaderaGruesa+VacioGavetaLateral/2
                        -FrenteGavetaLateral/2 )*x
    }
    union       {
      difference  {
        box       {
          < -FrenteTrastero/2   , 0             ,
                -ProfundidadTrastero    >,
          < +FrenteTrastero/2   , AltoTrastero  ,
                +0                      >
        }
        box       {
          < -FrenteTrastero/2   , 0             ,
                -ProfundidadTrastero    >+MaderaGruesa*<+1,-1,-1>,
          < +FrenteTrastero/2   , AltoTrastero  ,
                +0                      >
                +MaderaGruesa*<-1,+0,-1>-MaderaMedia*y
        }
        object  {
          DecoradoCurvo
          rotate        90*y
          scale         <  1, 0.3*AltoTrastero, ProfundidadTrastero >
          translate     -0.5*x-ProfundidadTrastero*z+0.05*AltoTrastero*y
        }
        object  {
          DecoradoCurvo
          rotate        90*y
          scale         <  1, 0.3*AltoTrastero, ProfundidadTrastero >
          translate     -0.5*x-ProfundidadTrastero*z+0.35*AltoTrastero*y
        }
        object  {
          DecoradoCurvo
          rotate        90*y
          scale         <  1, 0.3*AltoTrastero, ProfundidadTrastero >
          translate     -0.5*x-ProfundidadTrastero*z+0.65*AltoTrastero*y
        }
      }
      box       {
        < -FrenteTrastero/2     , VacioPiso1    , -ProfundidadPiso      >,
        < +FrenteTrastero/2     , VacioPiso1+MaderaPiso , 0 >
      }
      box       {
        < -FrenteTrastero/2     , VacioPiso1+MaderaPiso+VacioPiso2      ,
                                        -ProfundidadPiso      >,
        < +FrenteTrastero/2     , VacioPiso1+2*MaderaPiso+VacioPiso2     ,
                                        0 >
      }
      // Laterales Puertita Superior
      #local AnchoPuertaCentral         = 310*L;
      box       {
        < -AnchoPuertaCentral/2 , +VacioPiso1   ,
          -ProfundidadPiso+GruesoGaveta         >,
        < -AnchoPuertaCentral/2+GruesoGaveta    ,
          +VacioPiso1+MaderaPiso+VacioPiso2     , -GruesoInterior  >
      }
      box       {
        < -AnchoPuertaCentral/2 , +VacioPiso1   ,
          -ProfundidadPiso+GruesoGaveta         >*<-1,+1,+1>,
        < -AnchoPuertaCentral/2+GruesoGaveta    ,
          +VacioPiso1+MaderaPiso+VacioPiso2     , -GruesoInterior  >*<-1,1,1>
      }
      // Puertita Superior
      union     {
        box       {
          <  0                    , +VacioPiso1   ,
            -ProfundidadPiso                     >,
          < +AnchoPuertaCentral   , +VacioPiso1+VacioPiso2+2*MaderaPiso,
            -ProfundidadPiso+GruesoGaveta         >
          translate       -AnchoPuertaCentral/2*x
        }
        #local FraccionDecorado         = 0.7;
        #local AltoPuertaInferior = 394*L;
        
        object          {
          DecoradoPuerta
          scale         AltoPuertaInferior*<1,0,1>*FraccionDecorado+
                ( VacioPiso2+2*MaderaPiso )*FraccionDecorado*y
          translate     VacioPiso1*y+
                ( 1-FraccionDecorado )*( VacioPiso2+2*MaderaPiso )/2*y
                -ProfundidadPiso*z
        }
        object          {
          DecoradoPuerta
          scale         AltoPuertaInferior*<1,0,1>*FraccionDecorado+
                ( VacioPiso2+2*MaderaPiso )*FraccionDecorado*y
          translate     VacioPiso1*y+
                ( 1-FraccionDecorado )*( VacioPiso2+2*MaderaPiso )/2*y
                -AnchoPuertaCentral/4*x+
                -ProfundidadPiso*z
        }
        object          {
          DecoradoPuerta
          scale         AltoPuertaInferior*<1,0,1>*FraccionDecorado+
                ( VacioPiso2+2*MaderaPiso )*FraccionDecorado*y
          translate     VacioPiso1*y+
                ( 1-FraccionDecorado )*( VacioPiso2+2*MaderaPiso )/2*y
                +AnchoPuertaCentral/4*x+
                -ProfundidadPiso*z
        }
        object       {
          Agarradera
          translate  -ProfundidadPiso*z
                     +3*AnchoPuertaCentral/8*x
                     +0.50*( VacioPiso2+2*MaderaPiso )*y
                     +VacioPiso1*y
        }
        translate       +AnchoPuertaCentral/2*x+ProfundidadPiso*z
                                -GruesoGaveta*z
        rotate          Central*y
        translate       -AnchoPuertaCentral/2*x-ProfundidadPiso*z
                        +GruesoGaveta*z
        texture         {
          TexturaBase
          translate     5
          // para agregar un cambio cualquiera a la textura
        }
      }
      // BalconcitoPuerta
      object    {
        BalconcitoPuerta
        scale   AnchoPuertaCentral
        translate       -AnchoPuertaCentral/2*y
        rotate  -90*z
        translate       ( VacioPiso1+VacioPiso2+2*MaderaPiso
                +0.12*AnchoPuertaCentral )*y
                -ProfundidadPiso*z+0.10*AnchoPuertaCentral*z
                +GruesoGaveta*z
      }
      translate ( AltoBase+AltoGavetero )*y
    }
    // Nivel Superior
    union       {
      difference        {
        box       {
          < -FrenteSuperior/2 , 0            , -ProfundidadSuperior      >,
          < +FrenteSuperior/2 , AltoSuperior , 0                     >
        }
        
        box     {
          < -FrenteSuperior/2 , 0            , -ProfundidadSuperior      >
                +MaderaGruesa*<+1,-1,+1>,
          < +FrenteSuperior/2 , AltoSuperior , 0                     >
                +MaderaGruesa*<-1,+1,-1>
        }
        union   {
          object  {
            DecoradoCurvo
            scale           <-1,+1,+1>
            rotate        -90*z
            translate     y
            scale         < FrenteSuperior/4*0.9, 2*AltoSuelo   , 1     >
            translate     +FrenteSuperior/4*0.9*x-2*ProfundidadBase*z-AltoSuelo*y
          }
          object  {
            DecoradoCurvo
            scale           <-1,+1,+1>
            rotate        -90*z
            translate     y
            scale         < FrenteSuperior/4*0.9, 2*AltoSuelo   , 1     >
            translate     -FrenteSuperior/2*0.9*x-2*ProfundidadBase*z-AltoSuelo*y
          }
          object  {
            DecoradoCurvo
            scale         <-1,+1,+1>
            rotate        -90*z
            translate     y
            scale         < FrenteSuperior/4*0.9, 2*AltoSuelo   , 1     >
            translate     -FrenteSuperior/4*0.9*x-2*ProfundidadBase*z-AltoSuelo*y
          }
          object  {
            DecoradoCurvo
            scale         <-1,+1,+1>
            rotate        -90*z
            translate     y
            scale         < FrenteSuperior/4*0.9, 2*AltoSuelo   , 1     >
            translate     -2*ProfundidadBase*z-AltoSuelo*y
          }
          object  {
            DecoradoCurvo
            scale         <-1,+1,+1>
            rotate        -90*z
            translate     y
            scale         < ProfundidadSuperior/2*0.9, 2*AltoSuelo   , 1     >
            translate     -0.5*z-AltoSuelo*y
            rotate        90*y
            translate     -ProfundidadSuperior/2*z
          }
          object  {
            DecoradoCurvo
            scale         <-1,+1,+1>
            rotate        -90*z
            translate     y
            scale         <-ProfundidadSuperior/2*0.9, 2*AltoSuelo   , 1     >
            translate     -0.5*z-AltoSuelo*y
            rotate        90*y
            translate     -ProfundidadSuperior/2*z
          }
          translate       ( -AltoSuelo+0.9*AltoSuperior )*y
        }
      }
      translate ( AltoBase+AltoPrincipal )*y+CorrimientoSuperiorZ*z
    }
    texture     {
      TexturaBase
      translate 1
      rotate    90*y
    }
  }
#end

#macro TSMB_Trastero1()          
  object        {
    TSMB_TrasteroSalaMaderaBarnizada
        ( TSMB_TexturaBaseTrasteroDefault,
          TSMB_TexturaMetalBaratoDefault,
          TSMB_TexturaMaderaBarataDefault, 
          0.05, 0.04, 0.10, 01, 02, 05, 01 )
  }
#end
