// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com
// Apdo. 964-1250 Escaz, San Jos, Costa Rica 
// dedicado a Jess y Mara...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 6 diciembre 2010
//                
// Mis dibujos estn en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY 
// El objeto creado por la macro es una ventana de garage.  Se ubica
// segun las siguientes coordenadas:
// Minimo <  -320,      0,  -35 >*L
// Maximo <   320,   1110,  106 >*L
// el origen se encuentra debajo del centro inferior del marco
// Si se elimina la reja metalica, las dimensiones seran
// Minimo <  -320,      0,  -35 >*L
// Maximo <   320,   1110,   35 >*L
//
// El programador dispone de las siguientes definiciones
// "declare", para sus ajustes
// #declare WHFG_AnchoMarco                = 640*L;
// #declare WHFG_AltoMarco                 = 740*L;
// #declare WHFG_ProfundidadMarco          = 70*L;
// no modificables desde el exterior
// El radio del semicirculo superior seria simplemente
// "radio del semicirculo superior" = WHFG_AnchoMarco/2
// 
// Para crear el objeto, es probable se necesite perforar una
// pared.  Esto se puede realizar por diferencia con
// WHFG_ContainerDifferenceWHFG
// que contiene el marco de profundidad doble
// por ejemplo
// union        {
//   difference {
//      "ParedCustom..."
//      WHFG_ContainerDifferenceWHFG
//   }
//   WHFG_WindowHouseFrontGarage( 0.1, true )
// }
// La macro seria
// WHFG_WindowHouseFrontGarage( FraccionVidrio, BanderaReja )
//   FraccionVidrio       = de 0.0 a 1.0, cerrado... hasta 90 grados
//   BanderaReja          = true con rejas, false las elimina
//
// acepta los siguientes modificadores (si se indica es default) 
//   #declare WHFG_POV3_1                          = off;
//                      para asegurar compatibilidad con POVRay 3.1.
//                      Innecesario en versin actual.
//                      Se recomienda su uso para
//                      versiones futuras del objeto (activa con on).
//   #declare WHFG_FactorContainerDifference       = 1.0;
//                      permite aumentar la profundidad en pared
//                      de WHFG_ContainerDifferenceWHFG para
//                      paredes de mayor grosor.
//                      El valor default produce el doble del marco  
//   #declare WHFG_TexturaContainerDifference   = texture { ... } 
//                      permite aplicar una textura por diferencia
//                      cuando se perfora la pared
//                      por default no aplica ninguna textura
//   #declare WHFG_TexturaMarcoVentana          = texture { ... }
//                      se aplica en el marco 
//   #declare WHFG_TexturaVidrioVentana         = texture { ... }
//                      se aplica en el vidrio superior                             
//   #declare WHFG_TexturaVidrioRectangular     = texture { ... } 
//                      se aplica en las laminas rectangulares de vidrio
//   #declare WHFG_TexturaSujetaVidrio          = texture { ... }       
//                      se aplica en el soporte de las laminas
//                      rectangulares, directamente
//                      a ambos lados de los vidrio
//   #declare WHFG_TexturaSujetaSujetaVidrio    = texture { ... }
//                      soporte metalico en ambas laterales de los
//                      vidrios rectangulares
//   #declare WHFG_TexturaRejaMetalica          = texture { ... }
//                      textura para la reja
// Para usarlo es suficiente
// #include "WHFG.inc"
// object       {
//   WHFG_WindowHouseFrontGarage(0.0,true)
// }
//       

#include "textures.inc"
#include "stones1.inc" 
#include "stones2.inc"
#include "metals.inc"

#local   L                      = 0.001;

#ifndef ( WHFG_POV3_1 )
  #declare WHFG_POV3_1          = off;
#end 


#declare WHFG_AnchoMarco                = 640*L;
#declare WHFG_AltoMarco                 = 740*L;
#declare WHFG_ProfundidadMarco          = 70*L;

#ifndef ( WHFG_FactorContainerDifference )
  #declare WHFG_FactorContainerDifference       = 1.0;
#end

#declare WHFG_ContainerDifferenceWHFG     = union         {
  box       {
    < -WHFG_AnchoMarco/2,            0.0,
                -WHFG_FactorContainerDifference*WHFG_ProfundidadMarco >,
    <  WHFG_AnchoMarco/2, WHFG_AltoMarco,
                 WHFG_FactorContainerDifference*WHFG_ProfundidadMarco >
  }
  cylinder    {
    <       0.0,    WHFG_AltoMarco,
                -WHFG_FactorContainerDifference*WHFG_ProfundidadMarco >,
    <       0.0,    WHFG_AltoMarco,
                 WHFG_FactorContainerDifference*WHFG_ProfundidadMarco >,
    WHFG_AnchoMarco/2
  }
  #ifdef ( WHFG_TexturaContainerDifference )
    texture     {
      WHFG_TexturaContainerDifference
    }
  #end
}

#macro WHFG_WindowHouseFrontGarage( FraccionVidrio, BanderaReja )
// Datos generales del soporte de los vidrios
  #local   L                            = 0.001;
  #local AnchoMarco                     = WHFG_AnchoMarco;
  #local AltoMarco                      = WHFG_AltoMarco;
  #local ProfundidadMarco               = WHFG_ProfundidadMarco;
  #local ProfundidadMarcoInterno        = 30*L;
  #local ProfundidadMarcoInterno2       = 15*L;
  #local GruesoMarco                    = 14*L;
  #local GruesoMarcoInterno             = 7*L;
  #local GruesoMarcoInterno2            = 5*L;

// WHFG WindowHouseFrontGarage
  #ifndef ( WHFG_TexturaMarcoVentana )  
    #declare WHFG_TexturaMarcoVentana         = texture       {
      T_Grnt1
    }
  #end

// Datos generales del vidrio
  #local TotalRectanguloVidrio          = 11;
  #local ProfundidadRectanguloVidrio    = 2*L;
  #local AltoRectanguloVidrio           = 70*L;
  #local AnchoRectanguloVidrio          = 580*L;
  #local AnchoSujetaVidrio              = 5*L;
  #local ProfundidadSujetaVidrio        = 0.5*L;
  #local ProfundidadSujetaSujetaVidrio  = 8*ProfundidadSujetaVidrio
                +ProfundidadRectanguloVidrio;
  #local DesplazadoAlturaVidrio         =
                ( AltoMarco-2*GruesoMarco )/( TotalRectanguloVidrio );
  #local MenorAnguloVidrio              =
        degrees( asin( ProfundidadRectanguloVidrio/DesplazadoAlturaVidrio ) );
  #local MayorAnguloVidrio              = 90;
  #local AnguloVidrio                   = MenorAnguloVidrio+
                FraccionVidrio*( MayorAnguloVidrio-MenorAnguloVidrio );
  #ifndef ( WHFG_TexturaVidrioVentana )  
    #declare WHFG_TexturaVidrioVentana        = texture       {
      pigment   { rgbt <1,1,1,0.6> } 
      normal    {
        crackle 
        scale   0.05
      }
    }
  #end
  #ifndef ( WHFG_TexturaVidrioRectangular )
    #declare WHFG_TexturaVidrioRectangular    = texture       {
      pigment   { rgbt <1,1,1,0.4> }
      normal    {
        crackle 
        scale   0.02
      }
    }
  #end
  #ifndef ( WHFG_TexturaSujetaVidrio )  
    #declare WHFG_TexturaSujetaVidrio         = texture       {
      T_Brass_4B
    }
  #end
  #ifndef ( WHFG_TexturaSujetaSujetaVidrio )  
    #declare WHFG_TexturaSujetaSujetaVidrio   = texture       {
      T_Brass_2E
    }
  #end

// DatosUtilesVarios
  #local Dif                            = 0.1*L;
                                // corrector para objeto tipo  difference

// Datos generales de la reja metalica
  #local ProfundidadRejaMetalica        = 14*L;
  #local GruesoRejaMetalica             = 3.5*L;
  #local GruesoVarillaMetalica          = 6.5*L;
  #local AnguloVerticalVarillaSuperior  = 30;
  #local ProfundidadSoporteMetalico     = 57*L;
  #local AnchoBaseSoporteMetalico       = 25*L;
  #local AlturaPrimerSoporte            = 70*L;
  #local UbicacionSegundoSoporte        = 60*L;
  #local TotalDecoracionesReja          = 3;
  #local TotalBloquesVarilla            = 2*TotalDecoracionesReja-1;
  #local AlturaSegundoSoporte           =
                AltoMarco-UbicacionSegundoSoporte-ProfundidadRejaMetalica;
  #ifndef ( WHFG_TexturaRejaMetalica )
    #declare WHFG_TexturaRejaMetalica           = texture       {
      pigment           { color Gray20 }
    }
  #end

  #macro WHFG_VarillaReja( LargoVarilla )
    box         {
      < -GruesoVarillaMetalica/2,          0.0, -GruesoVarillaMetalica/2 >,
      < +GruesoVarillaMetalica/2, LargoVarilla, +GruesoVarillaMetalica/2 >
    }
  #end

  #local SoporteReja                    = union {
    box         {
      < 0.0, 0.0, 0.0 >,
      < GruesoRejaMetalica, ProfundidadRejaMetalica,
                                        -ProfundidadSoporteMetalico >
    }
    box         {
      < 0.0, 0.0, 0.0 >,
      < AnchoBaseSoporteMetalico-ProfundidadRejaMetalica/2,
                            ProfundidadRejaMetalica, -GruesoRejaMetalica >
    }
    cylinder    {
      < AnchoBaseSoporteMetalico-ProfundidadRejaMetalica/2,
                            ProfundidadRejaMetalica/2, 0.0 >,
      < AnchoBaseSoporteMetalico-ProfundidadRejaMetalica/2,
                            ProfundidadRejaMetalica/2, -GruesoRejaMetalica >,
      ProfundidadRejaMetalica/2
    }
    difference  {
      sphere    {
        < AnchoBaseSoporteMetalico-ProfundidadRejaMetalica/2,
                  ProfundidadRejaMetalica/2, ProfundidadRejaMetalica/7 >
        ProfundidadRejaMetalica/2
      }
      plane     {
        -z      ,0
      }
    }
  }

  #local MarcoVentana                   = union {
    // MarcoPrincipal
    difference  {
      box       {
        < -AnchoMarco/2,       0.0, -ProfundidadMarco >,
        <  AnchoMarco/2, AltoMarco,               0.0 >
      }
      box       {
        < -AnchoMarco/2+GruesoMarco,           GruesoMarco,
                                                -ProfundidadMarco >-Dif*z,
        <  AnchoMarco/2-GruesoMarco, AltoMarco-GruesoMarco,
                                                              0.0 >+Dif*z
      }
    }
    difference  {
      cylinder    {
        <       0.0,    AltoMarco,      -ProfundidadMarco >,
        <       0.0,    AltoMarco,                    0.0 >,
        AnchoMarco/2
      }
      cylinder    {
        <       0.0,    AltoMarco,                    0.0 >+Dif*z,
        <       0.0,    AltoMarco,      -ProfundidadMarco >-Dif*z,
        AnchoMarco/2-GruesoMarco
      }
      plane       {
        y, AltoMarco
      }
    }
    // Marco interno
    box         {
      < -AnchoMarco/2+GruesoMarco                   ,          0.0,
                        ( -ProfundidadMarco-ProfundidadMarcoInterno )/2 >,
      < -AnchoMarco/2+GruesoMarco+GruesoMarcoInterno,    AltoMarco,
                        ( -ProfundidadMarco+ProfundidadMarcoInterno )/2 >
    }
    box         {
      < +AnchoMarco/2-GruesoMarco-GruesoMarcoInterno,          0.0,
                        ( -ProfundidadMarco-ProfundidadMarcoInterno )/2 >,
      < +AnchoMarco/2-GruesoMarco                   ,    AltoMarco,
                        ( -ProfundidadMarco+ProfundidadMarcoInterno )/2 >
    }
    box         {
      < -AnchoMarco/2+GruesoMarco+GruesoMarcoInterno,          0.0,
                        ( -ProfundidadMarco-ProfundidadMarcoInterno2 )/2 >,
      < -AnchoMarco/2+GruesoMarco+GruesoMarcoInterno+GruesoMarcoInterno2,
                        AltoMarco,
                        ( -ProfundidadMarco+ProfundidadMarcoInterno2 )/2 >
    }
    box         {
      < +AnchoMarco/2-GruesoMarco-GruesoMarcoInterno-GruesoMarcoInterno2, 0.0,
                        ( -ProfundidadMarco-ProfundidadMarcoInterno2 )/2 >,
      < +AnchoMarco/2-GruesoMarco-GruesoMarcoInterno,    AltoMarco,
                        ( -ProfundidadMarco+ProfundidadMarcoInterno2 )/2 >
    }
    box         {
      < +AnchoMarco/2-GruesoMarco,                         AltoMarco,
                        ( -ProfundidadMarco-ProfundidadMarcoInterno )/2 >,
      < -AnchoMarco/2+GruesoMarco,      GruesoMarcoInterno+AltoMarco,
                        ( -ProfundidadMarco+ProfundidadMarcoInterno )/2 >
    }
    difference  {
      cylinder    {
        <       0.0,    AltoMarco,
                ( -ProfundidadMarco-ProfundidadMarcoInterno )/2 >,
        <       0.0,    AltoMarco,
                ( -ProfundidadMarco+ProfundidadMarcoInterno )/2 >,
        AnchoMarco/2-GruesoMarco
      }
      cylinder    {
        <       0.0,    AltoMarco,                    0.0 >,
        <       0.0,    AltoMarco,      -ProfundidadMarco >,
        AnchoMarco/2-GruesoMarco-GruesoMarcoInterno
      }
      plane       {
        y, AltoMarco
      }
    }
    texture     { WHFG_TexturaMarcoVentana }
  }

  #local VidrioVentana                  = union {
    difference  {
      cylinder    {
        <       0.0,    AltoMarco,
                ( -ProfundidadMarco-ProfundidadRectanguloVidrio )/2 >,
        <       0.0,    AltoMarco,
                ( -ProfundidadMarco+ProfundidadRectanguloVidrio )/2 >,
        AnchoMarco/2-GruesoMarco
      }
      plane     {
        y, AltoMarco
      }
    }
    #local CuentaVidrio                 = 0;
    #local DesplazadoAltura             = DesplazadoAlturaVidrio;
    #while ( CuentaVidrio < TotalRectanguloVidrio )
      union     {
        box       {
          < -AnchoRectanguloVidrio/2, -AltoRectanguloVidrio/2,
                                -ProfundidadRectanguloVidrio/2 >,
          < +AnchoRectanguloVidrio/2, +AltoRectanguloVidrio/2,
                                +ProfundidadRectanguloVidrio/2 >
          texture       {
            WHFG_TexturaVidrioRectangular
          }
        }
        difference        {
          union     {
            box       {
              < -AnchoRectanguloVidrio/2, -AltoRectanguloVidrio/2,
               ( -ProfundidadRectanguloVidrio-ProfundidadSujetaVidrio )/2 >,
              < -AnchoRectanguloVidrio/2+AnchoSujetaVidrio,
                        +AltoRectanguloVidrio/2,
               ( +ProfundidadRectanguloVidrio+ProfundidadSujetaVidrio )/2 >
              texture       { WHFG_TexturaSujetaVidrio }
            }
            box       {
              < +AnchoRectanguloVidrio/2, -AltoRectanguloVidrio/2,
               ( -ProfundidadRectanguloVidrio-ProfundidadSujetaVidrio )/2 >,
              < +AnchoRectanguloVidrio/2-AnchoSujetaVidrio,
                        +AltoRectanguloVidrio/2,
               ( +ProfundidadRectanguloVidrio+ProfundidadSujetaVidrio )/2 >
              texture       { WHFG_TexturaSujetaVidrio }
            }
          }
          box     {
            < -AnchoRectanguloVidrio/2, -AltoRectanguloVidrio/10,
               ( -ProfundidadRectanguloVidrio-2*ProfundidadSujetaVidrio )/2 >,
            < +AnchoRectanguloVidrio/2,
                        +AltoRectanguloVidrio/10,
               ( +ProfundidadRectanguloVidrio+2*ProfundidadSujetaVidrio )/2 >
          }
        }
        rotate       AnguloVidrio*x
        translate  -ProfundidadMarco/2*z
                +( GruesoMarco+( 0.5+CuentaVidrio )*DesplazadoAltura )*y
      }
      #local CuentaVidrio               = CuentaVidrio+1;
    #end
    union       {
      box       {
        < -AnchoMarco/2+GruesoMarco+GruesoMarcoInterno,
        +GruesoMarco, ( -ProfundidadMarco-ProfundidadSujetaSujetaVidrio )/2 >,
        < ( -AnchoRectanguloVidrio )/2,
        +AltoMarco-GruesoMarco,
        ( +ProfundidadMarco+ProfundidadSujetaSujetaVidrio )/2 >
      }
      box       {
        < +AnchoMarco/2-GruesoMarco-GruesoMarcoInterno,
        +GruesoMarco, ( -ProfundidadMarco-ProfundidadSujetaSujetaVidrio )/2 >,
        < ( +AnchoRectanguloVidrio )/2,
        +AltoMarco-GruesoMarco,
        ( +ProfundidadMarco+ProfundidadSujetaSujetaVidrio )/2 >
      }
      texture   {
        WHFG_TexturaSujetaSujetaVidrio
      }
    }
    texture     {
      WHFG_TexturaVidrioVentana
    }
  }

  #local RejaMetalicaVentana            = union {
    // SoportePrincipal
    difference  {
      box       {
        < -AnchoMarco/2,       0.0, -ProfundidadRejaMetalica >,
        <  AnchoMarco/2, AltoMarco,                      0.0 >
      }
      box       {
        < -AnchoMarco/2+GruesoRejaMetalica,           GruesoRejaMetalica,
                                        -ProfundidadRejaMetalica >-Dif*z,
        <  AnchoMarco/2-GruesoRejaMetalica, AltoMarco-GruesoRejaMetalica,
                                                             0.0 >+Dif*z
      }
    }
    difference  {
      cylinder    {
        <       0.0,    AltoMarco,      -ProfundidadRejaMetalica >,
        <       0.0,    AltoMarco,                           0.0 >,
        AnchoMarco/2
      }
      cylinder    {
        <       0.0,    AltoMarco,                           0.0 >+Dif*z,
        <       0.0,    AltoMarco,      -ProfundidadRejaMetalica >-Dif*z,
        AnchoMarco/2-GruesoRejaMetalica
      }
      plane       {
        y, AltoMarco
      }
    }
    // SoporteReja
    union       {
      object      {
        SoporteReja
        translate         -AnchoMarco/2*x+AlturaPrimerSoporte*y
      }
      object      {
        SoporteReja
        translate         -AnchoMarco/2*x+AlturaSegundoSoporte*y
      }
      object      {
        SoporteReja
        rotate    180*z
        translate +AnchoMarco/2*x
                        +( AlturaPrimerSoporte+ProfundidadRejaMetalica )*y
      }
      object      {
        SoporteReja
        rotate    180*z
        translate +AnchoMarco/2*x
                        +( AlturaSegundoSoporte+ProfundidadRejaMetalica )*y
      }
      translate  ProfundidadSoporteMetalico*z
    }
    union       {
      #local CuentaVarillaVertical        = 0;
      #local DesplazadoVarilla            =
                        AnchoMarco/( TotalBloquesVarilla+1 );

      // calculo varilla oblicua rectangulo principal
      #local AnguloVarillaOblicuaRectangulo     =
        degrees( atan2( ( 2*DesplazadoVarilla-2*GruesoVarillaMetalica ),
                                                AltoMarco ) );
      #local LargoVarillaOblicuaRectangulo      =
                ( DesplazadoVarilla-GruesoVarillaMetalica )
                /sin( radians( AnguloVarillaOblicuaRectangulo ) );

      #while  ( CuentaVarillaVertical < TotalBloquesVarilla )
        object    {
          WHFG_VarillaReja( AltoMarco )
          translate       -ProfundidadRejaMetalica/2*z
            +( -AnchoMarco/2+DesplazadoVarilla*( CuentaVarillaVertical+1 ) )*x
        }
        #if ( ( CuentaVarillaVertical/2
                        -int( CuentaVarillaVertical / 2 ) ) = 0 )
          object        {
            WHFG_VarillaReja( LargoVarillaOblicuaRectangulo )
            rotate       AnguloVarillaOblicuaRectangulo*z
            translate       -ProfundidadRejaMetalica/2*z
              +( -AnchoMarco/2-GruesoVarillaMetalica/2
                        +DesplazadoVarilla*( CuentaVarillaVertical+1 ) )*x
              +AltoMarco/2*y
          }
          object        {
            WHFG_VarillaReja( LargoVarillaOblicuaRectangulo )
            rotate      -AnguloVarillaOblicuaRectangulo*z
            translate       -ProfundidadRejaMetalica/2*z
              +( -AnchoMarco/2+GruesoVarillaMetalica/2
                        +DesplazadoVarilla*( CuentaVarillaVertical+1 ) )*x
              +AltoMarco/2*y
          }
          object        {
            WHFG_VarillaReja( LargoVarillaOblicuaRectangulo )
            rotate      ( 180-AnguloVarillaOblicuaRectangulo )*z
            translate       -ProfundidadRejaMetalica/2*z
              +( -AnchoMarco/2-GruesoVarillaMetalica/2
                        +DesplazadoVarilla*( CuentaVarillaVertical+1 ) )*x
              +AltoMarco/2*y
          }
          object        {
            WHFG_VarillaReja( LargoVarillaOblicuaRectangulo )
            rotate      -( 180-AnguloVarillaOblicuaRectangulo )*z
            translate       -ProfundidadRejaMetalica/2*z
              +( -AnchoMarco/2+GruesoVarillaMetalica/2
                        +DesplazadoVarilla*( CuentaVarillaVertical+1 ) )*x
            +AltoMarco/2*y
          }
        #end
        #local CuentaVarillaVertical      = CuentaVarillaVertical+1;
      #end
    }
    difference  {
      union       {
        #local CuentaVarillaVertical        = 0;
        #while  ( CuentaVarillaVertical < TotalBloquesVarilla )
          object    {
            WHFG_VarillaReja( AltoMarco )
            #if ( CuentaVarillaVertical < ( TotalDecoracionesReja-1 ) )
              rotate        +AnguloVerticalVarillaSuperior*z
            #else
              #if ( CuentaVarillaVertical > ( TotalDecoracionesReja-1 ) )
                rotate      -AnguloVerticalVarillaSuperior*z
              #end
            #end
            translate       -ProfundidadRejaMetalica/2*z
              +( -AnchoMarco/2+DesplazadoVarilla*( CuentaVarillaVertical+1 ) )*x
              +AltoMarco*y
          }
          // Faltan todavia dos varillas en la parte superior centrales
          object    {
            WHFG_VarillaReja( AltoMarco )
            rotate        +AnguloVerticalVarillaSuperior*z
            translate       -ProfundidadRejaMetalica/2*z+AltoMarco*y
          }
          object    {
            WHFG_VarillaReja( AltoMarco )
            rotate        -AnguloVerticalVarillaSuperior*z
            translate       -ProfundidadRejaMetalica/2*z+AltoMarco*y
          }
          #local CuentaVarillaVertical      = CuentaVarillaVertical+1;
        #end
      }
      sphere      {
        -ProfundidadRejaMetalica/2*z+AltoMarco*y,
        AnchoMarco/2
        inverse
      }
    }
    translate   ( -ProfundidadMarco-ProfundidadSoporteMetalico )*z
    texture     {
      WHFG_TexturaRejaMetalica
    }
  }

  union        {
    object     { MarcoVentana }
    object     { VidrioVentana }
    #if ( BanderaReja )
      object     { RejaMetalicaVentana }
    #end
    translate   +0.5*WHFG_ProfundidadMarco*z
  }
#end
